=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2008-11-06 23:00:22 +0000
+++ bzrlib/repofmt/pack_repo.py	2008-11-11 00:03:17 +0000
@@ -53,6 +53,7 @@
     errors,
     lockable_files,
     lockdir,
+    revision as _mod_revision,
     symbol_versioning,
     )
 
@@ -2163,6 +2164,15 @@
         # make it raise to trap naughty direct users.
         raise NotImplementedError(self._iter_inventory_xmls)
 
+    def _find_revision_outside_set(self, revision_ids):
+        revision_set = frozenset(revision_ids)
+        for revid in revision_ids:
+            parent_ids = self.get_parent_map([revid]).get(revid, ())
+            for parent in parent_ids:
+                if parent not in revision_set:
+                    return parent
+        return _mod_revision.NULL_REVISION
+        
     def fileids_altered_by_revision_ids(self, revision_ids, _inv_weave=None):
         """Find the file ids and versions affected by revisions.
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2008-11-06 23:00:22 +0000
+++ bzrlib/tests/test_repository.py	2008-11-11 00:03:17 +0000
@@ -32,6 +32,7 @@
                            UnsupportedFormatError,
                            )
 from bzrlib import graph
+from bzrlib.branchbuilder import BranchBuilder
 from bzrlib.btree_index import BTreeBuilder, BTreeGraphIndex
 from bzrlib.index import GraphIndex, InMemoryGraphIndex
 from bzrlib.repository import RepositoryFormat
@@ -687,6 +688,64 @@
             repo.chk_bytes.keys())
 
 
+class TestDevelopment3FindRevisionOutsideSet(TestCaseWithTransport):
+    """Tests for _find_revision_outside_set."""
+
+    def setUp(self):
+        super(TestDevelopment3FindRevisionOutsideSet, self).setUp()
+        self.builder = self.make_branch_builder('source', format='development3')
+        self.builder.start_series()
+        self.builder.build_snapshot('initial', None,
+            [('add', ('', 'tree-root', 'directory', None))])
+        self.repo = self.builder.get_branch().repository
+        self.addCleanup(self.builder.finish_series)
+        
+    def assertRevisionOutsideSet(self, expected_result, rev_set):
+        self.assertEqual(
+            expected_result, self.repo._find_revision_outside_set(rev_set))
+
+    def test_simple(self):
+        self.builder.build_snapshot('revid1', None, [])
+        self.builder.build_snapshot('revid2', None, [])
+        rev_set = ['revid2']
+        self.assertRevisionOutsideSet('revid1', rev_set)
+
+    def test_not_first_parent(self):
+        self.builder.build_snapshot('revid1', None, [])
+        self.builder.build_snapshot('revid2', None, [])
+        self.builder.build_snapshot('revid3', None, [])
+        rev_set = ['revid3', 'revid2']
+        self.assertRevisionOutsideSet('revid1', rev_set)
+
+    def test_not_null(self):
+        rev_set = ['initial']
+        self.assertRevisionOutsideSet(_mod_revision.NULL_REVISION, rev_set)
+
+    def test_not_null_set(self):
+        self.builder.build_snapshot('revid1', None, [])
+        rev_set = [_mod_revision.NULL_REVISION]
+        self.assertRevisionOutsideSet(_mod_revision.NULL_REVISION, rev_set)
+
+    def test_ghost(self):
+        self.builder.build_snapshot('revid1', None, [])
+        rev_set = ['ghost', 'revid1']
+        self.assertRevisionOutsideSet('initial', rev_set)
+
+    def test_ghost_parent(self):
+        self.builder.build_snapshot('revid1', None, [])
+        self.builder.build_snapshot('revid2', ['revid1', 'ghost'], [])
+        rev_set = ['revid2', 'revid1']
+        self.assertRevisionOutsideSet('initial', rev_set)
+
+    def test_righthand_parent(self):
+        self.builder.build_snapshot('revid1', None, [])
+        self.builder.build_snapshot('revid2a', ['revid1'], [])
+        self.builder.build_snapshot('revid2b', ['revid1'], [])
+        self.builder.build_snapshot('revid3', ['revid2a', 'revid2b'], [])
+        rev_set = ['revid3', 'revid2a']
+        self.assertRevisionOutsideSet('revid2b', rev_set)
+
+
 class TestWithBrokenRepo(TestCaseWithTransport):
     """These tests seem to be more appropriate as interface tests?"""
 

